unit main;

////////////////////////////////////////////////////////////////////////////////
//
// Emerald example front end.
//
// This interface is an example of how the Emerald object can be used
// to create an encryption program.  All of the major properties of the
// Emerald object are used in this example, including progress bars.
//
// Steve Corbett
// Dec 1 1998
// Updated Jan 8 1999
//
////////////////////////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, PersistentForm, uEmerald, Buttons, JPEG;

type
  TfmMain = class(TFormPersistent)
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Emerald: TEmerald;
    imgTitle: TImage;
    lbCopyright: TLabel;
    pnControlCluster: TPanel;
    lbInputFileName: TLabel;
    lbOutputFileName: TLabel;
    editOutputFileName: TEdit;
    editInputFileName: TEdit;
    btnBrowseInputFile: TButton;
    btnBrowseOutputFile: TButton;
    rbPassword: TRadioButton;
    editPasswords: TEdit;
    btnBrowseKeyFile: TButton;
    editKeyFile: TEdit;
    rbKeyFile: TRadioButton;
    lbOperation: TLabel;
    pnBottom: TPanel;
    cbOperation: TComboBox;
    btnPreferences: TButton;
    btnHelp: TButton;
    btnExit: TButton;
    btnAbout: TButton;
    btnExecute: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnBrowseKeyFileClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure rbKeyFileClick(Sender: TObject);
    procedure rbPasswordClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure btnBrowseInputClick(Sender: TObject);
    procedure btnBrowseOutputClick(Sender: TObject);
    procedure btnPreferencesClick(Sender: TObject);
    procedure btnExecuteClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
  private

    KeyFileIdx: Integer;
  end;

var
  fmMain: TfmMain;

implementation

uses progress, preferences, Registry;

{$R *.DFM}

procedure TfmMain.FormCreate(Sender: TObject);
var
  Reg: TRegistry;
begin
  // Prepare the Emerald object:
  ProgressForm := TProgressForm.Create(nil);
  Emerald.MajorProgressBar := ProgressForm.ProgressMajor;
  Emerald.MinorProgressBar := ProgressForm.ProgressMinor;
  cbOperation.ItemIndex := 0; // encrypt

  // Get preferences from registry:
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  if not Reg.OpenKey('\Software\Secret Portal\Emerald',False) then
  begin
    // Create key and assume defaults:
    Reg.OpenKey('\Software\Secret Portal\Emerald',True);
    Reg.WriteInteger('KeyFileIdx',2);
    Reg.WriteBool('KeyValidation',True);
  end;

  // Read preferences:
  try
    KeyFileIdx := Reg.ReadInteger('KeyFileIdx');
    Emerald.KeyValidation := Reg.ReadBool('KeyValidation');
  except
    KeyFileIdx := 2;
    Emerald.KeyValidation := True;
  end;

  // Close up:
  Reg.CloseKey;
  Reg.Free;
end;

procedure TfmMain.FormShow(Sender: TObject);
begin
  // Get form left and top:
  Load;

  // Accept any command line parameters:
  if ParamCount > 0 then
  begin
    // If it's an Emerald file, go into decrypt mode.
    if (UpperCase(ExtractFileExt(ParamStr(1))) = '.EMR') or
       (UpperCase(ExtractFileExt(ParamStr(1))) = '.EMD') then
      cbOperation.ItemIndex := 1;

    // Otherwise, stay in encrypt mode (default).
    editInputFileName.Text := ParamStr(1);
  end;
end;

procedure TfmMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Reg: TRegistry;
begin
  // Save form left and top:
  Save;

  // Store preferences:
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  Reg.OpenKey('\Software\Secret Portal\Emerald',True);
  Reg.WriteInteger('KeyFileIdx',KeyFileIdx);
  Reg.WriteBool('KeyValidation',Emerald.KeyValidation);
  Reg.CloseKey;
  Reg.Free;

  ProgressForm.Free;
end;

procedure TfmMain.btnBrowseKeyFileClick(Sender: TObject);
begin
  // Browse for a key file:
  OpenDialog.DefaultExt := 'emk';
  OpenDialog.Filter := 'Emerald key files (*.emk)|*.emk|Any file (*.*)|*.*';
  OpenDialog.FileName := '';
  if not OpenDialog.Execute then Exit;
  editKeyFile.Text := OpenDialog.FileName;
end;

procedure TfmMain.btnAboutClick(Sender: TObject);
begin
  MessageDlg('Emerald v6.4  Copyright 1994-1998 Steve Corbett, All Rights Reserved.'+chr(13)+chr(13)+
        'Visit the official Emerald web site: http://www.secretportal.com/Emerald/'+chr(13)+
        'This site will contain any upgrades or add-ons as they become available.',
        mtInformation,[mbOK],0);
end;

procedure TfmMain.rbKeyFileClick(Sender: TObject);
begin
  editPasswords.Enabled := False;
  editPasswords.Visible := False;
  editKeyFile.Enabled := True;
  editKeyFile.Visible := True;
  btnBrowseKeyFile.Enabled := True;
  btnBrowseKeyFile.Visible := True;
end;

procedure TfmMain.rbPasswordClick(Sender: TObject);
begin
  editPasswords.Enabled := True;
  editPasswords.Visible := True;
  editKeyFile.Enabled := False;
  editKeyFile.Visible := False;
  btnBrowseKeyFile.Enabled := False;
  btnBrowseKeyFile.Visible := False;
end;

procedure TfmMain.btnExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmMain.btnBrowseInputClick(Sender: TObject);
begin
  // Are we browsing for a file to encrypt or decrypt?
  case cbOperation.ItemIndex of
    0: begin // encrypt
      OpenDialog.Filter := 'Any file (*.*)|*.*';
      OpenDialog.Title := 'Select file to encrypt';
      OpenDialog.DefaultExt := '';
    end;

    1: begin // decrypt
      OpenDialog.Filter := 'Emerald Encrypted file (*.emr)|*.emr|Any file (*.*)|*.*';
      OpenDialog.Title := 'Select file to decrypt';
      OpenDialog.DefaultExt := 'emr';
    end;
  end;

  // Go:
  if not OpenDialog.Execute then Exit;
  editInputFileName.Text := OpenDialog.FileName;
end;

procedure TfmMain.btnBrowseOutputClick(Sender: TObject);
begin
  // Are we saving an encrypted or a decrypted file?
  case cbOperation.ItemIndex of
    0: begin // Encrypted
      SaveDialog.Filter := 'Emerald Encrypted file (*.emr)|*.emr|Any file (*.*)|*.*';
      SaveDialog.Title := 'Select output file';
      SaveDialog.DefaultExt := 'emr';
      SaveDialog.FileName := 'Encrypted.emr';
    end;

    1: begin // Decrypted
      SaveDialog.Filter := 'Any file (*.*)|*.*';
      SaveDialog.Title := 'Select output file';
      SaveDialog.DefaultExt := '';
      SaveDialog.FileName := '';
    end;
  end;

  // Go:
  if not SaveDialog.Execute then Exit;
  editOutputFileName.Text := SaveDialog.FileName;
end;

procedure TfmMain.btnPreferencesClick(Sender: TObject);
begin
  // Allocate memory:
  fmPreferences := TfmPreferences.Create(nil);

  // Set up the form according to current settings:
  fmPreferences.cbKeyFileSecurity.ItemIndex := KeyFileIdx;
  fmPreferences.rbKeyValidation.Checked := Emerald.KeyValidation;
  fmPreferences.rbNoKeyValidation.Checked := not Emerald.KeyValidation;

  // Allow user to change settings:
  if fmPreferences.ShowModal = mrOk then
  begin
    // Accept new settings:
    KeyFileIdx := fmPreferences.cbKeyFileSecurity.ItemIndex;
    Emerald.KeyValidation := fmPreferences.rbKeyValidation.Checked;
  end;

  // Release memory.
  fmPreferences.Free;
end;

procedure TfmMain.btnExecuteClick(Sender: TObject);
var
  Fubar: Boolean;
begin
  Fubar := False;

  // Validity checks:
  if editInputFileName.Text = '' then
  begin
    MessageDlg('You must supply a source file.',mtInformation,[mbOK],0);
    Exit;
  end;
  if (editPasswords.Text = '') and (editKeyFile.Text = '') then
  begin
    MessageDlg('You must supply at least one password or a key file.',mtInformation,[mbOK],0);
    Exit;
  end;

  // Set up the Emerald object:
  Emerald.InFileName := editInputFileName.Text;
  Emerald.OutFileName := editOutputFileName.Text;
  if rbPassword.Checked then Emerald.Password := editPasswords.Text;
  if rbKeyFile.Checked then Emerald.KeyFile := editKeyFile.Text;

  // More validity checks:
  if (cbOperation.ItemIndex = 1) and (not Emerald.IsEmeraldEncrypted) then
  begin
    MessageDlg('The format of the input file is not recognized.  This application only supports Emerald Encrypted files that were created with Emerald v6.3 or higher.',mtInformation,[mbOK],0);
    Exit;
  end;

  // If decrypting, and the key is in the encrypted file, check it against
  // whatever the user entered.
  if (cbOperation.ItemIndex = 1) and (Emerald.KeyValidationWasUsed) then
    if not Emerald.KeyIsValid then
    begin
      MessageDlg('The password(s) you entered do not match the one(s) in the file.',mtInformation,[mbOK],0);
      Exit;
    end;

  // Disable appropriate buttons:
  btnBrowseInputFile.Enabled := False;
  btnBrowseOutputFile.Enabled := False;
  btnExecute.Enabled := False;
  ProgressForm.ProgressMajor.Position := 0;
  ProgressForm.ProgressMinor.Position := 0;
  if cbOperation.ItemIndex = 0 then
    ProgressForm.Caption := 'Emerald v6.4 - Encryption in progress'
  else
    ProgressForm.Caption := 'Emerald v6.4 - Decryption in progress';
  ProgressForm.Show;

  // Go:
  try
    try
      if cbOperation.ItemIndex = 0 then
        Emerald.Encrypt
      else
        Emerald.Decrypt(False); // key validation already done.
    except
      on E:Exception do
      begin
        MessageDlg('An error occurred during the operation.  Message as follows:'+chr(13)+chr(13)+E.Message,mtError,[mbOk],0);
        Fubar := True; // prevent the "complete" dialog from popping up
      end;
    end;
  finally
    // Restore the form to user input enabled mode:
    btnBrowseInputFile.Enabled := True;
    btnBrowseOutputFile.Enabled := True;
    btnExecute.Enabled := True;
    ProgressForm.ProgressMajor.Position := 0;
    ProgressForm.ProgressMinor.Position := 0;
    ProgressForm.Hide;

    // Echo:
    if not Fubar then MessageDlg('Operation completed successfully.',mtInformation,[mbOK],0);
  end;

end;

procedure TfmMain.btnHelpClick(Sender: TObject);
begin
  MessageDlg('For information regarding how to use Emerald Encryption, please refer to the Emerald online help web site: '+chr(13)+
    'http://www.secretportal.com/Emerald/help.html',mtInformation,[mbOK],0);
end;

end.

